require 'exchange_rates/engine'
module ExchangeRates
  class Rate < ::ActiveRecord::Base
    self.table_name = 'exchange_rates'
    BASE_CURRENCY                   = 'HKD'.freeze
    CUSTOMS_VALUE_FALLBACK_CURRENCY = 'USD'.freeze
    # NB: to update the list of not supported currencies, run:
    # Country.all.map(&:currency_code).uniq - Money::Currency.table.map{|c| c[0].to_s.upcase}
    NOT_SUPPORTED_CURRENCIES        = [nil, '', 'IMP', 'TVD'].freeze

    validates :from_currency,         presence: true, length: { is: 3 }, format: { with: /\A[A-Z]+\z/ }
    validates :rate_to_base_currency, presence: true, numericality: { greater_than: 0 }

    def self.convert(amount, from_currency, to_currency = BASE_CURRENCY)
      case
      when amount == 0
        0
      when to_currency == from_currency
        amount
      when to_currency == BASE_CURRENCY
        amount * get_rate(from_currency)
      else
        amount * (get_rate(from_currency) / get_rate(to_currency))
      end
    end

    def self.get_rate(currency)
      return 1 if currency == BASE_CURRENCY
      cached = Rails.cache.fetch("#{currency.downcase}_to_#{BASE_CURRENCY.downcase}", expires_in: 1.day, race_condition_ttl: 5) do
        where(from_currency: currency).first.rate_to_base_currency
      end
      cached || where(from_currency: currency).first.rate_to_base_currency
    end
  end
end
